﻿using System;
using System.ServiceModel;
using System.ServiceModel.Web;
using TestClientApp.TestService;

namespace TestClientApp
{
	[ServiceContract]
	interface ITestService
	{
		[OperationContract]
		[WebGet(UriTemplate = "{category}/TestService.svc?value={value}")]
		string TestMethod1(string category, string value);

		[OperationContract]
		[WebInvoke(UriTemplate = "TestService.svc")]
		string TestMethod2(string arg);
	}

	class ClientApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Advanced Web Programming Model Test Client...");

			Console.WriteLine("Call the service using SOAP endpoint");
			using (TestServiceClient proxy1 = new TestServiceClient("BasicHttpBinding_ITestService")) {
				string result1 = proxy1.TestMethod1("MyCategory1", "MyValue1");
				Console.WriteLine("  TestMethod1 Result: {0}", result1);
				string result2 = proxy1.TestMethod2("Data1");
				Console.WriteLine("  TestMethod2 Result: {0}", result2);
			}

			Console.WriteLine("Call the service using Web endpoint");
			WebHttpBinding binding = new WebHttpBinding();
			Uri address = new Uri("http://simplework/wcf/example/web/advanced/web");
			WebChannelFactory<ITestService> factory = new WebChannelFactory<ITestService>(
				binding, address);
			ITestService proxy2 = factory.CreateChannel();
			using (proxy2 as IDisposable) {
				string result1 = proxy2.TestMethod1("MyCategory2", "MyValue2");
				Console.WriteLine("  TestMethod1 Result: {0}", result1);
				string result2 = proxy2.TestMethod2("Data2");
				Console.WriteLine("  TestMethod2 Result: {0}", result2);
			}
		}
	}
}
