﻿using System;
using System.ServiceModel;
using System.Data;
using System.Data.SqlClient;
using System.Transactions;

namespace TxServiceApp
{
	[ServiceContract(SessionMode = SessionMode.Required)]
	interface ITxService
	{
		[OperationContract]
		int InsertData(int val);

		[OperationContract]
		void SetComplete();

		[OperationContract]
		DataSet GetData();
	}

	[ServiceBehavior(TransactionAutoCompleteOnSessionClose = false, InstanceContextMode = InstanceContextMode.PerCall)]
	class ManualTxService : ITxService, IDisposable
	{
		[OperationBehavior(TransactionScopeRequired = true, TransactionAutoComplete = false)]
		public int InsertData(int val)
		{
			string connStr = "SERVER=(local);Trusted_Connection=true;DATABASE=TestDB";
			string query = "INSERT INTO TestTable VALUES(@val, 'TempData'); " +
						"SELECT @@IDENTITY";
			SqlConnection conn = new SqlConnection(connStr);
			SqlCommand cmd = new SqlCommand(query, conn);

			conn.Open();
			cmd.Parameters.Add("@val", SqlDbType.Int, 4).Value = val;
			try {
				decimal id = (decimal)cmd.ExecuteScalar();
				Console.WriteLine("New data id = {0}", id);
				return (int)id;
			}
			finally {
				conn.Close();
			}
		}

		[OperationBehavior(TransactionScopeRequired = true, TransactionAutoComplete = false)]
		public void SetComplete()
		{
			OperationContext.Current.SetTransactionComplete();
		}

		public DataSet GetData()
		{
			string connStr = "SERVER=(local);Trusted_Connection=true;DATABASE=TestDB";
			string query = "SELECT * FROM TestTable";
			SqlDataAdapter adapter = new SqlDataAdapter(query, connStr);
			DataSet ds = new DataSet();

			adapter.Fill(ds);
			return ds;
		}

		#region IDisposable Members

		public ManualTxService()
		{
			Console.WriteLine("+++ Object created. Id={0}, Ambient Transaction={1}",
				this.GetHashCode(), Transaction.Current != null);
		}

		void IDisposable.Dispose()
		{
			Console.WriteLine("--- Object disposed. Id={0}, Ambient Transaction={1}",
				this.GetHashCode(), Transaction.Current != null);
		}

		#endregion
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Manual Transaction Service Sample......");

			ServiceHost host = new ServiceHost(typeof(ManualTxService));
			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
