﻿using System;
using System.Transactions;

namespace NestedTransactionScopeTest
{
	class Program
	{
		static void Main(string[] args)
		{
			try {
				BadCodePattern.Method1();
			}
			catch (Exception ex) {
				Console.WriteLine(ex.Message);
			}

			try {
				GoodPattern.Method1();
			}
			catch (Exception ex) {
				Console.WriteLine(ex.Message);
			}
		}
	}


	class BadCodePattern
	{
		public static void Method1()
		{
			using(TransactionScope scope1 = new TransactionScope(
												 TransactionScopeOption.Required)) {
				try {
					Method2();
				}
				catch (Exception ex) {
					Console.WriteLine(ex.Message);
				}
				// scope2가 이미 트랜잭션 취소에 투표했으므로 추가적인 트랜잭션은
				// TransactionAbortedException 예외를 유발한다.
				Method3();
				scope1.Complete();
			}
		}

		public static void Method2()
		{
			using (TransactionScope scope2 = new TransactionScope(
												 TransactionScopeOption.Required)) {
				int a = 5;
				int b = 0;
				int c = a / b;				// Divide by 0 예외 발생
				scope2.Complete();
			}
		}

		public static void Method3()
		{
			using(TransactionScope scope3 = new TransactionScope(
												 TransactionScopeOption.Required)) {
				scope3.Complete();
			}
		}
	}

	class GoodPattern
	{
		public static void Method1()
		{
			using (TransactionScope scope1 = new TransactionScope(
												 TransactionScopeOption.Required)) {
				Method2();
				Method3();
				scope1.Complete();
			}
		}

		public static void Method2()
		{
			using (TransactionScope scope2 = new TransactionScope(
												 TransactionScopeOption.Required)) {
				int a = 5;
				int b = 0;
				int c = a / b;				// Divide by 0 예외 발생
				scope2.Complete();
			}
		}

		public static void Method3()
		{
			using (TransactionScope scope3 = new TransactionScope(
												 TransactionScopeOption.Required)) {
				scope3.Complete();
			}
		}
	}
}
