﻿using System;
using System.Collections.Generic;
using System.Transactions;
using System.Data;
using System.Data.SqlClient;

namespace SimpleTransaction
{
	class TxTestApp
	{
		const string ConnStr = "SERVER=(local);UID=Tester;PWD=test";
		const string ConnStr2 = "SERVER=(local);UID=Tester;PWD=test;DATABASE=Northwind";

		static void Main(string[] args)
		{
			Console.WriteLine("Simple Transaction Test using System.Transaction...");

			//CreateTestTable();
			//SimpleBasicTransaction();
			SimpleBasicTransactionWithTxOptions();
			//TransactionPromotionTest();
		}

		static void CreateTestTable()
		{
			Console.WriteLine("Creating test table...");

			string ddl = "IF EXISTS (SELECT * FROM sysobjects WHERE id = object_id('dbo.TestTable') and sysstat & 0xf = 3)\n" +
						"DROP TABLE dbo.TestTable\n" +
						"CREATE TABLE dbo.TestTable (\n" +
						"PK int IDENTITY(1,1) PRIMARY KEY,\n" +
						"COL1 int NULL,\n" +
						"COL2 nvarchar(64) NULL)\n";
			SqlConnection conn = new SqlConnection(ConnStr);
			SqlCommand cmd = new SqlCommand(ddl, conn);

			conn.Open();
			cmd.ExecuteNonQuery();
			conn.Close();

			Console.WriteLine("   Done...");
		}

		static void SimpleBasicTransaction()
		{
			Console.WriteLine("Simple Transaction Test (Local Transaction)");

			string query = "INSERT INTO TestTable VALUES(99, 'TempData'); " +
						"SELECT @@IDENTITY";
			using (TransactionScope scope = new TransactionScope()) {
				SqlConnection conn = new SqlConnection(ConnStr);
				SqlCommand cmd = new SqlCommand(query, conn);

				conn.Open();
				decimal id = (decimal)cmd.ExecuteScalar();
				Console.WriteLine("New data id = {0}", id);
				conn.Close();
				scope.Complete();
			}
			Console.WriteLine("   Done...");
		}

		static void SimpleBasicTransactionWithTxOptions()
		{
			Console.WriteLine("Simple Transaction Test with Transaction Option");

			string query = "INSERT INTO TestTable VALUES(99, 'TempData'); " +
						"SELECT @@IDENTITY";
			TransactionOptions options = new TransactionOptions();
			TransactionScopeOption txOption = TransactionScopeOption.Required;
			options.IsolationLevel = System.Transactions.IsolationLevel.ReadCommitted;
			options.Timeout = TimeSpan.FromSeconds(120);

			using (TransactionScope scope = new TransactionScope(txOption, options)) {
				SqlConnection conn = new SqlConnection(ConnStr);
				SqlCommand cmd = new SqlCommand(query, conn);

				conn.Open();
				decimal id = (decimal)cmd.ExecuteScalar();
				Console.WriteLine("New data id = {0}", id);
				conn.Close();
				scope.Complete();
			}
			Console.WriteLine("   Done...");
		}

		static void TransactionPromotionTest()
		{
			Console.WriteLine("Transaction Promotion Test (Distirbuted Transaction)");

			string query = "INSERT INTO TestTable VALUES(99, 'TempData'); " +
						"SELECT @@IDENTITY";
			string query2 = "SELECT COUNT(*) FROM Products";
			using (TransactionScope scope = new TransactionScope()) {
				SqlConnection conn = new SqlConnection(ConnStr);
				SqlCommand cmd = new SqlCommand(query, conn);

				conn.Open();
				decimal id = (decimal)cmd.ExecuteScalar();
				Console.WriteLine("New data id = {0}", id);
				conn.Close();


				SqlConnection conn2 = new SqlConnection(ConnStr2);
				SqlCommand cmd2 = new SqlCommand(query2, conn2);

				conn2.Open();
				int count = (int)cmd2.ExecuteScalar();
				Console.WriteLine("Products data count = {0}", count);
				conn2.Close();

				scope.Complete();
			}
			Console.WriteLine("   Done...");
		}
	}
}
