﻿using System;
using System.ServiceModel;
using System.ServiceModel.MsmqIntegration;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Channels;

namespace MsmqService
{
	[ServiceContract]
	[ServiceKnownType(typeof(string))]
	interface IMsmqService
	{
		[OperationContract(IsOneWay = true, Action="*")]
		void OnMessage(MsmqMessage<string> msg);
	}

	class MsmqService : IMsmqService
	{
		public void  OnMessage(MsmqMessage<string> msg)
		{
			string data = (string)msg.Body;
			Console.WriteLine("Message body = {0}", data);
		}
	}

	// MSMQ 기반 서비스는 응답을 제공하지 않기 때문에
	// 메시지를 deserialize 하는 등의 오류가 발생하면 예외 메시지가 표시되지 않는다.
	// 따라서 커스텀 예외 핸들러를 작성하여 예외 메시지를 처리한다.
	class ErrorHandler : IErrorHandler
	{
		public bool HandleError(Exception error)
		{
			Console.WriteLine("--- Error !!!! --------------------------------");
			Console.WriteLine(error.ToString());
			Console.WriteLine("-----------------------------------------------");
			return false;
		}

		public void ProvideFault(Exception error, MessageVersion version, ref Message fault)
		{
		}
	}

	// 커스텀 예외 핸들러 설치를 위한 ServiceHost 타입 선언
	class MyServiceHost : ServiceHost
	{
		public MyServiceHost(Type serviceType, Uri baseAddress) : base(serviceType, baseAddress)
		{
		}

		protected override void InitializeRuntime()
		{
 			base.InitializeRuntime();
			ChannelDispatcher dispatcher = (ChannelDispatcher)base.ChannelDispatchers[0];
			dispatcher.ErrorHandlers.Add(new ErrorHandler());
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("MSMQ Integration Sample with WCF......");

			MsmqIntegrationBinding binding = new MsmqIntegrationBinding();
			binding.Security.Mode = MsmqIntegrationSecurityMode.None;

			MyServiceHost host = new MyServiceHost(typeof(MsmqService),
				new Uri("msmq.formatname:DIRECT=OS:.\\private$\\generictxqueue"));
			host.AddServiceEndpoint(typeof(IMsmqService), binding, "");

			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
