﻿using System;
using System.IO;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.ServiceModel.Security;
using System.IdentityModel.Selectors;
using System.IdentityModel.Tokens;
using System.Security.Principal;

namespace SecureServiceApp
{
	[ServiceContract]
	interface ISecureService
	{
		[OperationContract]
		void SecureMethod(string arg);
	}

	class SecureService : ISecureService
	{
		public void SecureMethod(string arg)
		{
			Console.WriteLine("SecureMethod invoked... arg={0}", arg);
			ServiceSecurityContext ctx = OperationContext.Current.ServiceSecurityContext;
			if (ctx.IsAnonymous) {
				Console.WriteLine("  Anonymous client");
			}
			else {
				Console.WriteLine("  User = {0}", ctx.PrimaryIdentity.Name);
				Console.WriteLine("  AuthenticateType = {0}", ctx.PrimaryIdentity.AuthenticationType);
			}
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Message Security - Certificate Authentication Test Service......");

			BasicHttpBinding binding = new BasicHttpBinding();
			binding.Security.Mode = BasicHttpSecurityMode.Message;
			binding.Security.Message.ClientCredentialType = 
				BasicHttpMessageCredentialType.Certificate;

			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;

			//X509Certificate2 serviceCert = new X509Certificate2("..\\..\\..\\..\\Certs\\localhost.pfx", "1234");

			FileStream stream = new FileStream("..\\..\\..\\..\\Certs\\localhost.pfx", FileMode.Open);
			byte[] rawData = new byte[stream.Length];
			stream.Read(rawData, 0, rawData.Length);
			stream.Close();
			X509Certificate2 serviceCert = new X509Certificate2(rawData, "1234");

			ServiceHost host = new ServiceHost(typeof(SecureService),
				new Uri("http://localhost/wcf/example/security/certauth"));
			host.AddServiceEndpoint(typeof(ISecureService), binding, "");
			host.Description.Behaviors.Add(behavior);
			host.Credentials.ServiceCertificate.Certificate = serviceCert;
			host.Credentials.ClientCertificate.Authentication.CertificateValidationMode =
				X509CertificateValidationMode.Custom;
			host.Credentials.ClientCertificate.Authentication.CustomCertificateValidator =
				new CustomX509CertificateValidator();

			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}
	}

	class CustomX509CertificateValidator : X509CertificateValidator
	{
		public override void Validate(X509Certificate2 certificate)
		{
			// check it is issued from a trusted CA
			//X509CertificateValidator.ChainTrust.Validate(certificate);

			if (this.CheckValidUsingDB(certificate.SerialNumber) == false) {
				throw new SecurityTokenValidationException("Unknown user certificate");
			}
		}

		private bool CheckValidUsingDB(string serialNumber)
		{
			// 간단한 예제이므로 TestUser 인증서만 허용한다.
			if (serialNumber == "710E82F5BD73EEA44B1188B167696840") {
				return true;
			}
			return false;
		}
	}

}
