﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Security.Principal;

namespace SecureServiceApp
{
	[ServiceContract]
	interface ISecureService
	{
		[OperationContract]
		void SecureMethod(string arg);
		[OperationContract]
		void SecureMethod2(string arg);
	}

	class SecureService : ISecureService
	{
		[OperationBehavior(Impersonation = ImpersonationOption.Allowed)]
		public void SecureMethod(string arg)
		{
			Console.WriteLine("SecureMethod invoked... arg={0}", arg);
			ShowServiceSecurityInfo();
		}

		[OperationBehavior(Impersonation = ImpersonationOption.Required)]
		public void SecureMethod2(string arg)
		{
			Console.WriteLine("SecureMethod2 invoked... arg={0}", arg);
			ShowServiceSecurityInfo();
		}

		private static void ShowServiceSecurityInfo()
		{
			ServiceSecurityContext ctx = ServiceSecurityContext.Current;
			Console.WriteLine("  Primary Identity = {0}", ctx.PrimaryIdentity.Name);
			WindowsIdentity identity = WindowsIdentity.GetCurrent();
			Console.WriteLine("  Current Windows Account = {0}", identity.Name);
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Client Impersonation Test Service......");

			WindowsIdentity identity = WindowsIdentity.GetCurrent();
			Console.WriteLine("Process Windows Account = {0}", identity.Name);

			NetTcpBinding binding = new NetTcpBinding();
			binding.Security.Mode = SecurityMode.Transport;
			binding.Security.Transport.ClientCredentialType = 
				TcpClientCredentialType.Windows;

			ServiceHost host = new ServiceHost(typeof(SecureService),
				new Uri("net.tcp://localhost:8000/wcf/example/security/impersonate"));
			host.AddServiceEndpoint(typeof(ISecureService), binding, "");

			ServiceMetadataBehavior behavior1 = new ServiceMetadataBehavior();
			host.Description.Behaviors.Add(behavior1);
			host.AddServiceEndpoint(typeof(IMetadataExchange),
				MetadataExchangeBindings.CreateMexTcpBinding(), "mex");

			//ServiceAuthorizationBehavior behavior2 =
			//    host.Description.Behaviors.Find<ServiceAuthorizationBehavior>();
			//if (behavior2 == null) {
			//    behavior2 = new ServiceAuthorizationBehavior();
			//    host.Description.Behaviors.Add(behavior2);
			//}
			//behavior2.ImpersonateCallerForAllOperations = true;

			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
