﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Security.Cryptography.X509Certificates;

namespace SecureServiceApp
{
	[ServiceContract]
	interface ISecureService
	{
		[OperationContract]
		void SecureMethod(string arg);
	}

	class SecureService : ISecureService
	{
		public void SecureMethod(string arg)
		{
			Console.WriteLine("SecureMethod invoked... arg={0}", arg);
			ServiceSecurityContext ctx = OperationContext.Current.ServiceSecurityContext;
			if (ctx.IsAnonymous) {
				Console.WriteLine("  Anonymous client");
			}
			else {
				Console.WriteLine("  User = {0}", ctx.PrimaryIdentity.Name);
				Console.WriteLine("  AuthenticateType = {0}", ctx.PrimaryIdentity.AuthenticationType);
			}
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("HTTP Certificate Authentication Test Service......");

			BasicHttpBinding binding = new BasicHttpBinding();
			binding.Security.Mode = BasicHttpSecurityMode.Transport;
			binding.Security.Transport.ClientCredentialType =
				HttpClientCredentialType.Certificate;

			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpsGetEnabled = true;

			ServiceHost host = new ServiceHost(typeof(SecureService),
				new Uri("https://localhost/wcf/example/security/certauth"));
			host.AddServiceEndpoint(typeof(ISecureService), binding, "");
			host.Description.Behaviors.Add(behavior);
			host.Credentials.ServiceCertificate.SetCertificate(
				"CN=localhost", StoreLocation.LocalMachine, StoreName.My);

			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
