﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.Net.Security;
using System.ServiceModel.Description;

namespace SecureServiceApp
{
	[ServiceContract(ProtectionLevel = ProtectionLevel.None)]
	interface ISecureService
	{
		[OperationContract]
		void SecureMethod(string arg);
	}

	[ServiceBehavior(IncludeExceptionDetailInFaults=true)]
	class SecureService : ISecureService
	{
		public void SecureMethod(string arg)
		{
			Console.WriteLine("SecureMethod invoked... arg={0}", arg);
			ServiceSecurityContext ctx = OperationContext.Current.ServiceSecurityContext;
			if (ctx == null || ctx.IsAnonymous) {
				Console.WriteLine("  Anonymous client");
			}
			else {
				Console.WriteLine("  User = {0}", ctx.PrimaryIdentity.Name);
				Console.WriteLine("  AuthenticateType = {0}", ctx.PrimaryIdentity.AuthenticationType);
			}
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("NamedPipe Transport Security Test Service...");

			NetNamedPipeBinding binding = new NetNamedPipeBinding();
			binding.Security.Mode = NetNamedPipeSecurityMode.Transport;			// default
			//binding.Security.Transport.ProtectionLevel = ProtectionLevel.Sign;

			ServiceHost host = new ServiceHost(typeof(SecureService),
				new Uri("net.pipe://localhost/wcf/example/security/pipe"));
			host.AddServiceEndpoint(typeof(ISecureService), binding, "");

			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			host.Description.Behaviors.Add(behavior);
			host.AddServiceEndpoint(typeof(IMetadataExchange),
				MetadataExchangeBindings.CreateMexNamedPipeBinding(), "mex");

			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
