﻿using System;
using System.ServiceModel;
using BindingExtension;
using System.ServiceModel.Description;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel.Channels;
using System.IdentityModel.Selectors;
using System.ServiceModel.Security;
using System.IdentityModel.Tokens;

namespace BindingTestServiceApp
{
	[ServiceContract]
	interface ITestService
	{
		[OperationContract]
		void TestMethod();
	}

	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	class TestService : ITestService
	{
		public void TestMethod()
		{
			Console.WriteLine("TestMethod invoked......");
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Basic CustomBinding Test Service......");

			CustomHttpBinding binding = new CustomHttpBinding();

			ServiceHost host = new ServiceHost(typeof(TestService),
				new Uri("http://localhost/wcf/example/custombinding/custom"));
			host.AddServiceEndpoint(typeof(ITestService), binding, "");

			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;
			host.Description.Behaviors.Add(behavior);

			host.Credentials.ServiceCertificate.SetCertificate(
				"CN=localhost", StoreLocation.LocalMachine, StoreName.My);
			host.Credentials.UserNameAuthentication.UserNamePasswordValidationMode =
				UserNamePasswordValidationMode.Custom;
			host.Credentials.UserNameAuthentication.CustomUserNamePasswordValidator =
				new CustomUserNameValidator();

			host.Open();
			Console.WriteLine("The service started... Press any key to stop...");
			Console.ReadKey(true);
			host.Close();
		}
	}

	class CustomUserNameValidator : UserNamePasswordValidator
	{
		public override void Validate(string userName, string password)
		{
			// check the username and password.
			if (userName != "TempUser" || password != "1234") {
				throw new SecurityTokenException("Unknown user.");
			}
			Console.WriteLine("{0} is a valid user.", userName);
		}
	}
}
