﻿using System;
using System.ServiceModel;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Channels;

namespace UserInfoExtension
{
	// 사용자 정보를 전달하고 수신하는 message inspector 구현
	class UserInfoMessageInspector : IClientMessageInspector, IDispatchMessageInspector
	{
		#region "IClientMessageInspector 인터페이스 구현"

		// 채널 계층에 Message 객체를 전송하기 전에 호출된다.
		public object BeforeSendRequest(ref Message request, IClientChannel channel)
		{
			UserInfo user = UserInfo.Current;
			if (user != null) {
				MessageHeader userInfoHeader = MessageHeader.CreateHeader(
					"UserInfo", "urn:UserInfo", user);
				request.Headers.Add(userInfoHeader);
			}
			return null;
		}

		public void AfterReceiveReply(ref Message reply, object correlationState)
		{
			// do nothing
		}

		#endregion

		#region "IDispatchMessageInspector 인터페이스 구현"

		public object AfterReceiveRequest(ref Message request, IClientChannel channel, InstanceContext instanceContext)
		{
			int index = request.Headers.FindHeader("UserInfo", "urn:UserInfo");
			UserInfo user = null;

			if (index >= 0) {
				user = request.Headers.GetHeader<UserInfo>(index);
			}

			UserInfo.Current = user;
			return null;
		}

		public void BeforeSendReply(ref Message reply, object correlationState)
		{
			// do nothing
		}

		#endregion
	}
}
