﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Runtime.Serialization;

namespace CollectionDataContractService
{
	[ServiceContract(Namespace = "http://noname.com/collection")]
	interface ICollectionTest
	{
		[OperationContract]
		string[] GetStrings();
		[OperationContract]
		CustomType[] GetCustomTypes();

		[OperationContract]
		IEnumerable GetSimpleObjects();
		[OperationContract]
		[ServiceKnownType(typeof(CustomType))]
		IEnumerable GetCustomObjects();
		[OperationContract]
		IEnumerable<CustomType> GetCustomObjectsTyped();

		[OperationContract]
		[ServiceKnownType(typeof(CustomType))]
		ICollection GetCustomObjectCollection();
		[OperationContract]
		ICollection<CustomType> GetCustomObjectCollectionTyped();
		
		[OperationContract]
		IList GetStringList();
		[OperationContract]
		[ServiceKnownType(typeof(CustomType))]
		IList GetCustomObjectList();
		[OperationContract]
		IList<CustomType> GetCustomObjectListTyped();
		
		[OperationContract]
		ArrayList GetArrayList();
		[OperationContract]
		[ServiceKnownType(typeof(CustomType))]
		ArrayList GetCustomObjectArrayList();
		[OperationContract]
		List<CustomType> GetTypedList();
		
		[OperationContract]
		IDictionary GetDictionary();
		[OperationContract]
		[ServiceKnownType(typeof(CustomType))]
		Hashtable GetHashtable();
		[OperationContract]
		Dictionary<int, CustomType> GetDictionaryTyped();

		[OperationContract]
		[ServiceKnownType(typeof(CustomType))]
		MyDictionary1 GetCustomDictionary1();
		[OperationContract]
		MyDictionary2 GetCustomDictionary2();
	}

	[DataContract(Namespace = "http://noname.com/collection/data")]
	public struct CustomType
	{
		[DataMember]
		public int a;

		public CustomType(int val)
		{ a = val; }
	}

	[Serializable]
	public class MyDictionary1 : IDictionary
	{
		Hashtable _Dictionary = new Hashtable();

		#region IDictionary Members

		public void Add(object key, object value)
		{ _Dictionary.Add(key, value); }

		public void Clear()
		{ _Dictionary.Clear(); }

		public bool Contains(object key)
		{ return _Dictionary.Contains(key); }

		public IDictionaryEnumerator GetEnumerator()
		{ return _Dictionary.GetEnumerator(); }

		public bool IsFixedSize
		{ get { return _Dictionary.IsFixedSize; } }

		public bool IsReadOnly
		{ get { return _Dictionary.IsFixedSize; } }

		public ICollection Keys
		{ get { return _Dictionary.Keys; } }

		public void Remove(object key)
		{ _Dictionary.Remove(key); }

		public ICollection Values
		{ get { return _Dictionary.Values; } }

		public object this[object key]
		{
			get { return _Dictionary[key]; }
			set { _Dictionary[key] = value; }
		}
		#endregion

		#region ICollection Members

		public void CopyTo(Array array, int index)
		{ _Dictionary.CopyTo(array, index); }

		public int Count
		{ get { return _Dictionary.Count; } }

		public bool IsSynchronized
		{ get { return _Dictionary.IsSynchronized; } }

		public object SyncRoot
		{ get { return _Dictionary.SyncRoot; } }

		#endregion

		#region IEnumerable Members

		IEnumerator IEnumerable.GetEnumerator()
		{ return ((IEnumerable)_Dictionary).GetEnumerator(); }

		#endregion
	}

	[CollectionDataContract(Namespace = "http://nonmae.com/collection/data", 
		ItemName = "DictionaryItem", KeyName = "KeyInfo", ValueName = "ValueInfo")]
	public class MyDictionary2 : Dictionary<string, CustomType>
	{
	}

	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	class TestService : ICollectionTest
	{
		public string[] GetStrings()
		{ return new string[] { "Hello", "WCF", "World" }; }

		public CustomType[] GetCustomTypes()
		{ return new CustomType[] { new CustomType(0), new CustomType(1), new CustomType(2) }; }

		public IEnumerable GetSimpleObjects()
		{ return new string[] { "Hello", "WCF", "World" }; }

		public IEnumerable GetCustomObjects()
		{ return new CustomType[] { new CustomType(10), new CustomType(11), new CustomType(12) }; }

		public IEnumerable<CustomType> GetCustomObjectsTyped()
		{ return new CustomType[] { new CustomType(20), new CustomType(21), new CustomType(22) }; }

		public ICollection GetCustomObjectCollection()
		{ return new CustomType[] { new CustomType(30), new CustomType(31), new CustomType(32) }; }

		public ICollection<CustomType> GetCustomObjectCollectionTyped()
		{ return new CustomType[] { new CustomType(40), new CustomType(41), new CustomType(42) }; }

		public IList GetStringList()
		{
			ArrayList list = new ArrayList();
			list.Add("Hello");
			list.Add("WCF");
			list.Add("World");
			return list;
		}

		public IList GetCustomObjectList()
		{
			List<CustomType> list = new List<CustomType>();
			list.Add(new CustomType(100));
			list.Add(new CustomType(101));
			list.Add(new CustomType(102));
			return list;
		}

		public IList<CustomType> GetCustomObjectListTyped()
		{
			List<CustomType> list = new List<CustomType>();
			list.Add(new CustomType(100));
			list.Add(new CustomType(111));
			list.Add(new CustomType(112));
			return list;
		}

		public ArrayList GetArrayList()
		{
			ArrayList array = new ArrayList();
			array.Add("Hello");
			array.Add("WCF");
			array.Add("World");
			return array;
		}

		public ArrayList GetCustomObjectArrayList()
		{
			ArrayList array = new ArrayList();
			array.Add(new CustomType(120));
			array.Add(new CustomType(121));
			array.Add(new CustomType(122));
			return array;
		}

		public List<CustomType> GetTypedList()
		{
			List<CustomType> list = new List<CustomType>();
			list.Add(new CustomType(100));
			list.Add(new CustomType(111));
			list.Add(new CustomType(112));
			return list;
		}

		public IDictionary GetDictionary()
		{
			Hashtable dictionary = new Hashtable();
			dictionary.Add("key0", "Hello");
			dictionary.Add("key1", "WCF");
			dictionary.Add("key2", "World");
			return dictionary;
		}

		public Hashtable GetHashtable()
		{
			Hashtable table = new Hashtable();
			table.Add(0, new CustomType(1000));
			table.Add(1, new CustomType(1001));
			table.Add(2, new CustomType(1002));
			return table;
		}

		public Dictionary<int, CustomType> GetDictionaryTyped()
		{
			Dictionary<int, CustomType> dictionary = new Dictionary<int, CustomType>();
			dictionary.Add(0, new CustomType(1010));
			dictionary.Add(1, new CustomType(1011));
			dictionary.Add(2, new CustomType(1012));
			return dictionary;
		}

		public MyDictionary1 GetCustomDictionary1()
		{
			MyDictionary1 dictionary = new MyDictionary1();
			dictionary.Add("key0", new CustomType(10000));
			dictionary.Add("key1", new CustomType(10001));
			dictionary.Add("key2", new CustomType(10002));
			return dictionary;
		}

		public MyDictionary2 GetCustomDictionary2()
		{
			MyDictionary2 dictionary = new MyDictionary2();
			dictionary.Add("key0", new CustomType(10010));
			dictionary.Add("key1", new CustomType(10011));
			dictionary.Add("key2", new CustomType(10012));
			return dictionary;
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Collection Data Contract Service Example...");

			ServiceHost host = new ServiceHost(
				typeof(TestService),
				new Uri("http://localhost/wcf/example/testservice"));

			// 종점 추가
			host.AddServiceEndpoint(
				typeof(ICollectionTest),
				new BasicHttpBinding(),
				"");

			// WSDL 을 위한 HTTP GET enable
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;
			host.Description.Behaviors.Add(behavior);

			host.Open();
			Console.WriteLine("Press Any Key to Stop Service...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
