﻿/*
 * 이 예제를 수행시키기 위해서는 SQL Server 혹은 SQL Server Expression과
 * Northwind Database가 필요하다. SQL Server Expression 과 Northwind 데이터
 * 베이스는 Microsoft의 다운로드 사이트에서 무료로 다운로드 받을 수 있다.
 * 
 */
using System;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Runtime.Serialization;
using System.Data;
using System.Data.SqlClient;
using System.Xml.Serialization;
using System.Xml.Schema;
using System.Xml;
using System.IO;

namespace MiscDataContractService
{
	[DataContract(Namespace = "http://noname.com/misc/data")]
	enum MyEnum
	{
		[EnumMember(Value="FirstValue")]
		First = 1,
		[EnumMember(Value = "SecondValue")]
		Second,
		Third
	}

	[ServiceContract(Namespace = "http://noname.com/misc")]
	interface IMiscService
	{
		[OperationContract]
		MyEnum GetEnum();
		[OperationContract]
		DataSet GetDataSet();
		[OperationContract]
		DataTable GetDataTable();
		[OperationContract]
		ProductDataSet GetTypedDataSet();
		[OperationContract]
		ProductDataSet.ProductsDataTable GetTypedDataTable();
	}

	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	class TestService : IMiscService
	{
		public MyEnum GetEnum()
		{
			return MyEnum.First;
		}

		private void GetResultSet(DataSet ds, string query, string tableName)
		{
			string connString = "SERVER=(local);DATABASE=Northwind;UID=Tester;PWD=test";
			SqlConnection conn = new SqlConnection(connString);
			SqlCommand cmd = new SqlCommand(query, conn);
			SqlDataAdapter adapter = new SqlDataAdapter(cmd);

			adapter.Fill(ds, tableName);
		}

		public DataSet GetDataSet()
		{
			string query = "SELECT * FROM Products; SELECT * FROM Suppliers";
			DataSet ds = new DataSet("TestDataSet");
			
			GetResultSet(ds, query, "ResultSet");
			return ds;
		}

		public DataTable GetDataTable()
		{
			string query = "SELECT * FROM Products";
			DataSet ds = new DataSet("ProductDataSet");
			
			GetResultSet(ds, query, "Products");
			return ds.Tables[0];
		}

		public ProductDataSet GetTypedDataSet()
		{
			string query = "SELECT * FROM Products";
			ProductDataSet ds = new ProductDataSet();

			GetResultSet(ds, query, "Products");
			return ds;
		}

		public ProductDataSet.ProductsDataTable GetTypedDataTable()
		{
			//
			// DataSet의 직렬화에 문제가 있기 때문에 Typed DataTable을 곧바로
			// 반환하는 경우, DataSet을 통해 Typed DataTable을 생성하여 반환할 수 없다.
			// 만약 DataSet을 통해 생성된 Typed DataTable을 반환하면 클라이언트 측은
			// DataTable을 역직렬화 하지 못해서 빈 테이블이 반환되어 버린다.
			//
			// DataTable은 기본적으로 DataSet의 스키마 네임스페이스를 사용하도록 되어 있다.
			// 서비스에서 DataSet에 대해 Fill을 호출하여 Typed DataSet을 만들고 이 DataSet의
			// Typed DataTable을 반환하면 이 테이블은 DataSet의 네임스페이스를 사용하게 될 것이다.
			// 하지만 클라이언트측에서는 역직렬화를 위해 DataSet을 이용하지 않고 직접
			// Typed DataTable을 생성(new)하게 된다. 이 경우, Typed DataTable이 DataSet과
			// 연관되어 있지 않기 때문에 네임스페이스가 존재하지 않게 된다.
			// 따라서 서비스에서 반환된 XML 문서(Typed DataTable에 대한 XML)에 설정된
			// 네임스페이스와 클라이언트측의 DataTable의 네임스페이스가 동일하지 않아서
			// 클라이언트가 Typed DataTable을 역직렬화 할 수 없게 된다.
			//
			// 이러한 현상을 피해가는 방법은 서비스도 데이터테이블을 DataSet을 사용하지 않고
			// 직접 생성하는 것이다.
			//
			string query = "SELECT * FROM Products";
			string connString = "SERVER=(local);DATABASE=Northwind;UID=Tester;PWD=test";
			SqlConnection conn = new SqlConnection(connString);
			SqlCommand cmd = new SqlCommand(query, conn);
			SqlDataAdapter adapter = new SqlDataAdapter(cmd);
			ProductDataSet.ProductsDataTable dt = new ProductDataSet.ProductsDataTable();

			adapter.Fill(dt);
			return dt;
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Miscellaneous Data Contract Service Example...");

			ServiceHost host = new ServiceHost(
				typeof(TestService),
				new Uri("http://localhost/wcf/example/testservice"));

			// 종점 추가
			host.AddServiceEndpoint(
				typeof(IMiscService),
				new BasicHttpBinding(),
				"");

			// WSDL 을 위한 HTTP GET enable
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;
			host.Description.Behaviors.Add(behavior);

			host.Open();
			Console.WriteLine("Press Any Key to Stop Service...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
