﻿using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Runtime.Serialization;

namespace ServiceImplementation
{
	[DataContract(Namespace = "http://noname.com/bookstore/data")]
	public class BookInfo
	{
		[DataMember]
		public string Title;
		[DataMember(IsRequired = true)]
		public readonly string Isbn;
		[DataMember]
		public string Author;
		[DataMember]
		public int Price;
		// 이 필드는 클라이언트에게 알려지지 않는 필드이며
		// 직렬화에도 해당 사항이 없다.
		public double MarginRate;

		public BookInfo(string title, string isbn, string author, int price)
		{
			Title = title;
			Isbn = isbn;
			Author = author;
			Price = price;
			MarginRate = 0.25;
		}
	}

	// 서비스 계약 선언
	[ServiceContract(Namespace = "http://noname.com/bookstore")]
	public interface IBookStore
	{
		[OperationContract]
		[NetDataContractFormat]
		BookInfo GetBookInfo(string isbn);
		[OperationContract]
		int Order(string userid, string isbn, int amount);
	}

	// 서비스 타입 구현
	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	public class BookStoreService : IBookStore
	{
		private static Dictionary<string, BookInfo> _Books = new Dictionary<string, BookInfo>();
		private static int id = 0;

		static BookStoreService()
		{
			BookInfo info;

			info = new BookInfo("WCF 10일만에 정복", "000-001", "김정복", 40000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("10일동안 하면 개똥이 만큼 WCF 한다", "000-002", "김개똥", 42000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("WCF Programming", "000-003", "Mr. Programming", 53000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("Programming WCF", "000-004", "Mrs. Programming", 37000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("Learning WCF", "000-005", "Unknown", 18000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("WCF Step by Step", "000-006", "김철수", 20000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo(null, "000-007", null, 20000);
			_Books.Add(info.Isbn, info);
		}

		public BookInfo GetBookInfo(string isbn)
		{
			if (_Books.ContainsKey(isbn) == true) {
				return _Books[isbn];
			}
			else {
				return null;
			}
		}

		private int GetUnitPrice(string isbn)
		{
			return 35000;
		}

		// 도서 주문
		public int Order(string userid, string isbn, int amount)
		{
			int total = GetUnitPrice(isbn) * amount;
			id += 1;
			// 주문 처리 로직
			// (단순 출력만... -_-; )
			Console.WriteLine(
				"Book Order: orderid={0}\n   userid={1}, isbn={2}, amount={3}, total={4}",
				id, userid, isbn, amount, total);
			// 주문 ID 반환
			return id;
		}

		// 주문 취소
		public bool Cancel(int orderid)
		{
			// 주문 취소 로직
			// (단순 출력만... ^^; )
			Console.WriteLine("Cancel Order: orderid={0}", orderid);
			return true;
		}
	}
}
