﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.ServiceModel.Activation;
using System.Reflection;

namespace IISHostWeb
{
	//
	// 서비스 호스트 객체를 생성하는 커스텀 팩토리 클래스
	//
	public class CustomServiceHostFactory : ServiceHostFactory
	{
		protected override ServiceHost CreateServiceHost(Type serviceType, Uri[] baseAddresses)
		{
			ServiceHost host = new ServiceHost(serviceType, baseAddresses);

			// HTTP GET 을 활성화 한다.
			ServiceMetadataBehavior behavior =
				host.Description.Behaviors.Find<ServiceMetadataBehavior>();
			if (behavior == null) {
				behavior = new ServiceMetadataBehavior();
				host.Description.Behaviors.Add(behavior);
			}
			behavior.HttpGetEnabled = true;
			// Reflection을 이용하여 서비스 계약을 알아낸다.
			Type contractType = null;
			if (IsServiceContract(serviceType) == true) {
				// 클래스를 계약으로 사용하는 경우이다.
				contractType = serviceType;
			}
			else {
				// 서비스 타입이 구현하는 인터페이스를 검사한다.
				Type[] interfaces = serviceType.GetInterfaces();
				foreach (Type intf in interfaces) {
					if (IsServiceContract(intf) == true) {
						if (contractType != null) {
							// 여러 서비스 계약을 사용하는 서비스 타입은
							// 지원하지 않는다. Configuration 설정을 따르도록 한다.
							contractType = null;
							break;
						}
						else {
							contractType = intf;
						}
					}
				}
			}
			if (contractType != null) {
				// 바인딩을 구성한다.
				WSHttpBinding binding = new WSHttpBinding("CommonHttpBinding");
				// 서비스 종점을 추가 한다.
				// 여기서 추가되는 종점은 configuartion에 설정된 종점에
				// 추가적으로 설정되는 종점이다.
				host.AddServiceEndpoint(contractType, binding, "");
			}
			// 구성한 ServiceHost 객체를 반환한다.
			return host;
		}

		// 주어진 타입에 ServiceContract 특성이 명시되어 있나 확인한다.
		private bool IsServiceContract(Type type)
		{
			object[] attrs = type.GetCustomAttributes(typeof(ServiceContractAttribute), false);
			if (attrs != null && attrs.Length > 0) {
				return true;
			}
			return false;
		}
	}
}
