﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.ServiceModel.Channels;
using System.Collections.ObjectModel;

namespace MetadataDiscoverSample
{
	class SampleApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Discovering Service Metadata Sample......");

			// 메타데이터 클라이언트 객체 생성
			MetadataExchangeClient client = new MetadataExchangeClient(
			    new Uri("http://localhost/wcf/example/testservice?wsdl"),
			    MetadataExchangeClientMode.HttpGet);
			// 메타데이터 수집
			MetadataSet metadata = client.GetMetadata();
			// WSDL 메타데이터 importer 객체 생성
			MetadataImporter importer = new WsdlImporter(metadata);
			// 서비스 종점 목록 획득
			ServiceEndpointCollection endpoints = importer.ImportAllEndpoints();
			// 서비스 계약 목록 획득
			Collection<ContractDescription> contracts = importer.ImportAllContracts();

			// 서비스 계약 정보 표시
			Console.WriteLine("Contracts:");
			foreach (ContractDescription contract in contracts) {
				Console.WriteLine("  Name: {0}", contract.Name);
				Console.WriteLine("  Namespace: {0}", contract.Namespace);
				Console.WriteLine("  Methods:");
				foreach (OperationDescription desc in contract.Operations) {
					Console.WriteLine("        {0}", desc.Name);
				}
			}

			// 서비스 종점 정보 표시
			Console.WriteLine("Endpoints:");
			foreach (ServiceEndpoint endpoint in endpoints) {
				Console.WriteLine("  [{0}]", endpoint.Name);
				Console.WriteLine("    Contract: {0}", endpoint.Contract.Name);
				Console.WriteLine("    Binding: {0}", endpoint.Binding.Name);
				Console.WriteLine("    Address: {0}", endpoint.Address);
			}
		}
	}
}
