﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.ServiceModel.Channels;
using System.ServiceModel.Configuration;

namespace TestServiceApp
{
	// 서비스 계약 선언
	[ServiceContract(Namespace = "http://noname.com/testservice")]
	interface ITestService
	{
		[OperationContract]
		string Echo(string arg);
	}

	// 서비스 타입 구현
	class TestService : ITestService
	{
		public string Echo(string arg)
		{
			Console.WriteLine("TestSerivce.Echo invoked.... arg={0}", arg);
			return arg;
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Metadata Exchange Sample Service......");

			//ServiceInitUsingCode();
			ServiceInitUsingConfig();
		}

		// Configuration을 이용하여 서비스를 초기화
		private static void ServiceInitUsingConfig()
		{
			// ServiceHost 객체 생성
			ServiceHost host = new ServiceHost(typeof(TestService),
				new Uri("http://localhost/wcf/example/testservice"),
				new Uri("net.tcp://localhost/wcf/example/testservice"),
				new Uri("net.pipe://localhost/wcf/example/testservice"));

			// 서비스 시작
			host.Open();
			Console.WriteLine("Press Any key to Stop the Service......");
			Console.ReadKey(true);
			// 서비스 종료
			host.Close();
		}

		// 코드를 이용하여 서비스를 초기화
		private static void ServiceInitUsingCode()
		{
			// ServiceHost 객체 생성
			ServiceHost host = new ServiceHost(typeof(TestService),
				new Uri("http://loner/wcf/example/testservice"),
				new Uri("net.tcp://localhost/wcf/example/testservice"),
				new Uri("net.pipe://localhost/wcf/example/testservice"));

			// 바인딩 설정들
			WSHttpBinding wsHttpBinding = new WSHttpBinding();
			NetTcpBinding netTcpBinding = new NetTcpBinding();
			NetNamedPipeBinding netNamedPipeBinding = new NetNamedPipeBinding();
			wsHttpBinding.Security.Mode = SecurityMode.None;
			netTcpBinding.Security.Mode = SecurityMode.None;
			netNamedPipeBinding.Security.Mode = NetNamedPipeSecurityMode.None;

			// 종점 추가
			// Configuration에 설정된 종점들을 모두 제거하여 다음 종점 추가 코드가
			// 문제가 없도록 한다.
			host.Description.Endpoints.Clear();
			host.AddServiceEndpoint(typeof(ITestService), wsHttpBinding, "");
			host.AddServiceEndpoint(typeof(ITestService), netTcpBinding, "");
			host.AddServiceEndpoint(typeof(ITestService), netNamedPipeBinding, "");

			// CustomBinding을 이용한 MEX 종점 추가
			// TransportBindingElement 만을 추가하여 CustomBinding 객체를 작성하면 된다.
			/*
			CustomBinding mexTcpBinding = new CustomBinding(
				new TcpTransportBindingElement());
			host.AddServiceEndpoint(typeof(IMetadataExchange), mexTcpBinding, "mex");
			CustomBinding mexHttpBinding = new CustomBinding(
				new HttpTransportBindingElement());
			host.AddServiceEndpoint(typeof(IMetadataExchange), mexHttpBinding, "mex");
			CustomBinding mexNamedPipeBinding = new CustomBinding(
				new NamedPipeTransportBindingElement());
			host.AddServiceEndpoint(typeof(IMetadataExchange), mexNamedPipeBinding, "mex");
			 */

			// MetadataExchangeBindings 클래스의 정적 메쏘드를 이용하여
			// MEX 바인딩 생성
			host.AddServiceEndpoint(typeof(IMetadataExchange),
				MetadataExchangeBindings.CreateMexHttpBinding(), "mex");
			host.AddServiceEndpoint(typeof(IMetadataExchange),
				MetadataExchangeBindings.CreateMexTcpBinding(), "mex");
			host.AddServiceEndpoint(typeof(IMetadataExchange),
				MetadataExchangeBindings.CreateMexNamedPipeBinding(), "mex");

			// Metadata behavior 추가
			ServiceMetadataBehavior behavior =
				host.Description.Behaviors.Find<ServiceMetadataBehavior>();
			if (behavior == null) {
				behavior = new ServiceMetadataBehavior();
				host.Description.Behaviors.Add(behavior);
			}

			// 서비스 시작
			host.Open();
			Console.WriteLine("Press Any key to Stop the Service......");
			Console.ReadKey(true);
			// 서비스 종료
			host.Close();
		}
	}
}
