﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;


namespace ServiceDescriptionSample
{
	// 서비스 계약 선언
	[ServiceContract(Namespace = "http://noname.com/testservice")]
	interface ITestContract
	{
		// 테스트를 위해 이름과 Action 설정을 지정하였다.
		[OperationContract(Name = "NewMethod")]
		string MyMethod1(string arg);
		[OperationContract]
		void MyMethod2(int arg);
	}

	// 서비스 구현
	[ServiceBehavior(ConfigurationName="TestServiceConfig")]
	class TestService : ITestContract
	{
		public string MyMethod1(string arg)
		{
			return "MyMethod1";
		}

		public void MyMethod2(int arg)
		{
			// do nothing...
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Service Description Sample......");
			Console.WriteLine();

			ServiceHost host = new ServiceHost(typeof(TestService),
				new Uri("http://localhost/wcf/example/testservice"),
				new Uri("net.tcp://localhost/wcf/example/testservice"));

			// ServiceDescription을 이용한 서비스 관련 정보 표시
			ServiceDescription serviceDescription = host.Description;
			Console.WriteLine("Service Name : {0}", serviceDescription.Name);
			Console.WriteLine("  Namespace : {0}", serviceDescription.Namespace);
			Console.WriteLine("  Type : {0}", serviceDescription.ServiceType.FullName);
			Console.WriteLine("  Configuration Name : {0}", serviceDescription.ConfigurationName);

			BasicHttpBinding basicHttpBinding = new BasicHttpBinding();
			NetTcpBinding netTcpBinding = new NetTcpBinding();

			host.AddServiceEndpoint(typeof(ITestContract), basicHttpBinding, "");
			host.AddServiceEndpoint(typeof(ITestContract), netTcpBinding, "");

			// 서비스 메타데이터 Behavior 설정
			ServiceMetadataBehavior behavior =
				serviceDescription.Behaviors.Find<ServiceMetadataBehavior>();
			if (behavior == null) {
				behavior = new ServiceMetadataBehavior();
				serviceDescription.Behaviors.Add(behavior);
			}
			behavior.HttpGetEnabled = true;
			behavior.HttpGetUrl = new Uri("metadata", UriKind.Relative);
			Console.WriteLine("  WSDL Url : {0}", behavior.HttpGetUrl);

			// 서비스 종점 정보 표시
			Console.WriteLine("  Endpoints:");
			foreach (ServiceEndpoint endpoint in serviceDescription.Endpoints) {
				Console.WriteLine("    [{0}]", endpoint.Name);
				Console.WriteLine("      Contract:");
				Console.WriteLine("        Name : {0}", endpoint.Contract.Name);
				Console.WriteLine("        Namespace : {0}", endpoint.Contract.Namespace);
				Console.WriteLine("        Type : {0}", endpoint.Contract.ContractType.FullName);
				Console.WriteLine("        Methods:");
				foreach (OperationDescription desc in endpoint.Contract.Operations) {
					Console.WriteLine("          {0}", desc.Name);
				}
				Console.WriteLine("      Binding: {0}", endpoint.Binding.Name);
				Console.WriteLine("      Address: {0}", endpoint.Address);
			}

			host.Open();
			Console.WriteLine("Press Any Key to Stop service...");
			Console.ReadKey(true);

			host.Close();
		}
	}
}
