﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.Runtime.Serialization;

namespace WasHostWeb
{
	// 데이터 계약 선언
	[DataContract(Namespace = "http://noname.com/bookstore/data")]
	public class BookInfo
	{
		[DataMember]
		public string Title;
		[DataMember(IsRequired = true)]
		public readonly string Isbn;
		[DataMember]
		public string Author;
		[DataMember]
		public int Price;

		public BookInfo(string title, string isbn, string author, int price)
		{
			this.Title = title;
			this.Isbn = isbn;
			this.Author = author;
			this.Price = price;
		}
	}

	// 서비스 계약 선언
	[ServiceContract(Namespace = "http://noname.com/bookstore")]
	interface IBookStore
	{
		[OperationContract]
		IList<BookInfo> GetBookList();
		[OperationContract]
		int Order(string userid, string isbn, int amount);
	}

	// 서비스 타입 구현
	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	class BookStoreService : IBookStore
	{
		private static int _OrderId = 1;
		private static Dictionary<string, BookInfo> _Books = new Dictionary<string, BookInfo>();

		static BookStoreService()
		{
			BookInfo info;

			info = new BookInfo("WCF 10일만에 정복", "000-001", "김정복", 40000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("10일동안 하면 개똥이 만큼 WCF 한다", "000-002", "김개똥", 42000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("WCF Programming", "000-003", "Mr. Programming", 53000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("Programming WCF", "000-004", "Mrs. Programming", 37000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("Learning WCF", "000-005", "Unknown", 18000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("WCF Step by Step", "000-006", "김철수", 20000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("WCF", "000-007", "박통신", 20000);
			_Books.Add(info.Isbn, info);
		}

		public IList<BookInfo> GetBookList()
		{
			List<BookInfo> books = new List<BookInfo>();

			foreach (string isbn in _Books.Keys) {
				books.Add(_Books[isbn]);
			}
			return books;
		}

		public int Order(string userid, string isbn, int amount)
		{
			if (_Books.ContainsKey(isbn) == true) {
				// 주문 내용을 DB 혹은 파일 등에 기록한다. (생략... -_-;)
				return _OrderId++;
			}
			else {
				return 0;
			}
		}
	}
}
