﻿using System;
using System.ServiceModel;

using ExceptionClientApp.BadService;

namespace ExceptionClientApp
{
	class ClientApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Fault Contract Sample Client......");

			using (BadContractClient proxy = new BadContractClient("WSHttpBinding_IBadContract")) {
				try {
					proxy.ThrowException();
				}
				catch (FaultException<Exception> ex) {
					Console.WriteLine("Service Error : {0}", ex.Detail.Message);
					Console.WriteLine(ex.Detail.StackTrace);
				}
				catch (Exception ex) {
					Console.WriteLine("Unknown Exception !!!");
					Console.WriteLine(ex.ToString());
				}
			}

			Console.WriteLine("\n--------------------------------------------------------------------\n");

			using (BadContractClient proxy = new BadContractClient("NetTcpBinding_IBadContract")) {
				try {
					proxy.ThrowGeneralException(null);
				}
				catch (FaultException<NullReferenceException> ex) {
					Console.WriteLine("Service Error : {0}", ex.Detail.Message);
					Console.WriteLine(ex.Detail.StackTrace);
				}
				catch (FaultException<ArgumentException> ex) {
					Console.WriteLine("Service Error : {0}", ex.Detail.Message);
					Console.WriteLine(ex.Detail.StackTrace);
				}
				catch (FaultException<Exception> ex) {
					Console.WriteLine("Service Error : {0}", ex.Detail.Message);
					Console.WriteLine(ex.Detail.StackTrace);
				}
				catch (Exception ex) {
					Console.WriteLine("Unknown Exception !!!");
					Console.WriteLine(ex.ToString());
				}

				// Fault contract에 명시된 예외가 발생하면 채널이 폴트 상태가 되지 않는다.
				try {
					proxy.ThrowGeneralException("argument");
				}
				catch (FaultException<NullReferenceException> ex) {
					Console.WriteLine("Service Error : {0}", ex.Detail.Message);
					Console.WriteLine(ex.Detail.StackTrace);
				}
				catch (FaultException<ArgumentException> ex) {
					Console.WriteLine("Service Error : {0}", ex.Detail.Message);
					Console.WriteLine(ex.Detail.StackTrace);
				}
				catch (FaultException<Exception> ex) {
					Console.WriteLine("Service Error : {0}", ex.Detail.Message);
					Console.WriteLine(ex.Detail.StackTrace);
				}
				catch (Exception ex) {
					Console.WriteLine("Unknown Exception !!!");
					Console.WriteLine(ex.ToString());
				}
			}
		}
	}
}
