﻿using System;
using System.ServiceModel;

namespace ExceptionServiceApp
{
	[ServiceContract]
	interface IBadContract
	{
		[OperationContract]
		[FaultContract(typeof(Exception))]
		void ThrowException();

		[OperationContract]
		[FaultContract(typeof(NullReferenceException))]
		[FaultContract(typeof(ArgumentException))]
		[FaultContract(typeof(Exception))]
		void ThrowGeneralException(string arg);
	}

	class BadService : IBadContract
	{
		public void ThrowException()
		{
			Console.WriteLine("Throwing Exception");
			try {
				throw new Exception("Test Exception with Fault Contract");
			}
			catch (Exception ex) {
				throw new FaultException<Exception>(ex, ex.Message);
			}
		}

		public void ThrowGeneralException(string arg)
		{
			Console.WriteLine("Throwing Multiple Exception by argument");

			try {
				if (arg == null) {
					throw new NullReferenceException("arg is null");
				}
				if (arg.Length == 0) {
					throw new ArgumentException("argument.Length is 0");
				}
				DoSomething(arg);
			}
			catch (NullReferenceException ex) {
				throw new FaultException<NullReferenceException>(ex, ex.Message);
			}
			catch (ArgumentException ex) {
				throw new FaultException<ArgumentException>(ex, ex.Message);
			}
			catch (Exception ex) {
				throw new FaultException<Exception>(ex, ex.Message);
			}
		}

		private void DoSomething(string arg)
		{
			throw new Exception("It is not supported operation");
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Fault Contract Sample Service......");

			// 베이스 주소 및 종점 설정은 configuration 파일을 사용한다.
			ServiceHost host = new ServiceHost(typeof(BadService));

			host.Open();
			Console.WriteLine("Press any key to stop the service");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
