﻿using System;
using System.ServiceModel;

namespace PerCallServiceApp
{
	// 서비스 계약 선언
	[ServiceContract]
	interface IPerCall
	{
		[OperationContract]
		int TestMethod();
	}

	// PerCall 서비스 타입 구현
	[ServiceBehavior(InstanceContextMode = InstanceContextMode.PerCall)]
	class PerCallService : IPerCall, IDisposable
	{
		private int _CallCount;

		public PerCallService()
		{
			Console.WriteLine(".ctor()   : Id={0}, Session={1}",
				this.GetHashCode(), OperationContext.Current.SessionId);
			_CallCount = 0;
		}

		public void Dispose()
		{
			Console.WriteLine("Dispose() : Id={0}, Session={1}",
				this.GetHashCode(), OperationContext.Current.SessionId);
		}

		public int TestMethod()
		{
			_CallCount++;
			Console.WriteLine("*** TestMethod() : CallCount={0}", _CallCount);
			return _CallCount;
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("PerCall Service Test Sample......");

			// 서비스 호스트와 종점 설정은 configuration을 사용한다.
			ServiceHost host = new ServiceHost(typeof(PerCallService));
			host.Open();
			Console.WriteLine("Press any key to stop the service...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
