﻿using System;
using System.IO;
using System.Collections.Generic;
using System.ServiceModel;
using System.ServiceModel.Description;

namespace SessionfulServiceApp
{
	// 서비스 계약 선언
	[ServiceContract(ConfigurationName = "IEnumData", SessionMode = SessionMode.Required)]
	interface IEnumData
	{
		[OperationContract(IsInitiating = true, IsTerminating = false)]
		void OpenEnum();
		[OperationContract(IsInitiating = false, IsTerminating = false)]
		string[] GetNext();
		[OperationContract(IsInitiating = false, IsTerminating = false)]
		string[] GetPrevious();
		[OperationContract(IsInitiating = false, IsTerminating = true)]
		void CloseEnum();
	}

	// 서비스 타입 구현
	[ServiceBehavior(ConfigurationName="EnumDataService", IncludeExceptionDetailInFaults = true)]
	class EnumDataService : IEnumData
	{
		private const int DataCount = 20;
		private List<string> _Data;
		private int _Position;

		public void OpenEnum()
		{
			// 데이터를 미리 읽어 둔다.
			_Data = new List<string>(
				Directory.GetDirectories("C:\\Windows", "*"));
			_Data.Sort();
			_Position = -20;
			Console.WriteLine("OpenEnum(): position = {0}", _Position);
		}

		public string[] GetNext()
		{
			// 현재 위치로부터 다음 20개의 데이터를 반환한다.
			if (_Position + DataCount > _Data.Count) {
				return null;
			}
			_Position += DataCount;
			string[] result = GetData();
			Console.WriteLine("GetNext(): position = {0}", _Position);
			return result;
		}

		private string[] GetData()
		{
			int count = (_Data.Count - _Position) < DataCount ? (_Data.Count - _Position) : DataCount;
			string[] result = new string[DataCount];
			_Data.CopyTo(_Position, result, 0, count);
			return result;
		}

		public string[] GetPrevious()
		{
			// 현재 위치로부터 이전 20개의 데이터를 반환한다.
			if (_Position < DataCount) {
				return null;
			}
			_Position -= DataCount;
			string[] result = GetData();
			Console.WriteLine("GetPrev(): position = {0}", _Position);
			return result;
		}

		public void CloseEnum()
		{
			// 데이터 열거에 관련된 클린업 작업을 수행한다.
			_Data.Clear();
			Console.WriteLine("CloseEnum()");
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Operation Sequence Controll Example Service.....");

			// 서비스 호스트 및 종점은 configuration 설정을 따른다.
			ServiceHost host = new ServiceHost(typeof(EnumDataService));
			host.Open();
			Console.WriteLine("Press any key to stop the service");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
