﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;

namespace SessionfulServiceApp
{
	// 서비스 계약 선언
	[ServiceContract(SessionMode = SessionMode.Required)]
	interface ISessionful
	{
		[OperationContract]
		int TestMethod();
	}

	// 서비스 타입 구현
	class SessionfulService : ISessionful, IDisposable
	{
		private int _CallCount;

		private void ShowServiceSessionId(string fmt, params object[] args)
		{
			string msg = String.Format(fmt, args);
			Console.WriteLine(msg);

			OperationContext ctx = OperationContext.Current;
			if (ctx != null) {
				Console.WriteLine("   Session id = {0}", ctx.SessionId);
			}
			else {
				Console.WriteLine("   OperationContext is not available");
			}
		}

		public SessionfulService()
		{
			ShowServiceSessionId("SessionfulService instance created...");
			_CallCount = 0;
		}

		public void Dispose()
		{
			ShowServiceSessionId("SessionfulService instance disposed...");
		}

		public int TestMethod()
		{
			_CallCount++;
			ShowServiceSessionId("TestMethod(): call count = {0}", _CallCount);
			return _CallCount;
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Sessionful Service Session Id Test Sample......");

			// 서비스 호스트
			ServiceHost host = new ServiceHost(typeof(SessionfulService),
				new Uri("http://localhost/wcf/example/sessionfulservice"),
				new Uri("net.tcp://localhost/wcf/example/sessionfulservice"),
				new Uri("net.pipe://localhost/wcf/example/sessionfulservice")
				);
			// WSHttpBinding 설정 (보안 세션)
			WSHttpBinding wsHttpBinding = new WSHttpBinding();
			wsHttpBinding.Security.Mode = SecurityMode.Message;
			// WSDualHttpBinding 설정 (신뢰할 수 있는 세션)
			WSDualHttpBinding wsDualHttpBinding = new WSDualHttpBinding();
			wsDualHttpBinding.Security.Mode = WSDualHttpSecurityMode.None;
			// NetTcpBinding 설정 (트랜스포트 세션)
			NetTcpBinding netTcpBinding = new NetTcpBinding();
			netTcpBinding.Security.Mode = SecurityMode.None;
			netTcpBinding.ReliableSession.Enabled = false;
			// NetNamedPipeBinding 설정 (트랜스포트 세션)
			NetNamedPipeBinding netNamedPipeBinding = new NetNamedPipeBinding();
			netNamedPipeBinding.Security.Mode = NetNamedPipeSecurityMode.None;
			// 서비스 종점 설정
			host.AddServiceEndpoint(typeof(ISessionful), wsHttpBinding, "securesession");
			host.AddServiceEndpoint(typeof(ISessionful), wsDualHttpBinding, "reliablesession");
			host.AddServiceEndpoint(typeof(ISessionful), netTcpBinding, "");
			host.AddServiceEndpoint(typeof(ISessionful), netNamedPipeBinding, "");
			// 메타데이터 HTTP GET 활성화
			ServiceMetadataBehavior behavior =
				host.Description.Behaviors.Find<ServiceMetadataBehavior>();
			if (behavior == null) {
				behavior = new ServiceMetadataBehavior();
				host.Description.Behaviors.Add(behavior);
			}
			behavior.HttpGetEnabled = true;
			// 서비스 구동
			host.Open();
			Console.WriteLine("Press any key to stop the service...");
			Console.ReadKey(true);
			// 서비스 종료
			host.Close();
		}
	}
}
