﻿using System;
using System.ServiceModel;

namespace SingletonServiceApp
{
	// 서비스 계약 선언
	[ServiceContract]
	interface ISingleton
	{
		[OperationContract]
		int TestMethod();
	}

	// Singleton 서비스 타입 구현
	[ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
	class SingletonService : ISingleton, IDisposable
	{
		private int _CallCount;

		public SingletonService()
		{
			Console.Write(".ctor() : Id={0}, ", this.GetHashCode());
			if (OperationContext.Current == null) {
				Console.WriteLine("OperationContext.Current == null");
			}
			else {
				Console.WriteLine("Session={1}",
					this.GetHashCode(), OperationContext.Current.SessionId);
			}
			_CallCount = 0;
		}

		public void Dispose()
		{
			Console.Write("Dispose() : Id={0}, ", this.GetHashCode());
			if (OperationContext.Current == null) {
				Console.WriteLine("OperationContext.Current == null");
			}
			else {
				Console.WriteLine("Session={1}",
					this.GetHashCode(), OperationContext.Current.SessionId);
			}
		}

		public int TestMethod()
		{
			_CallCount++;
			Console.WriteLine("*** TestMethod() : CallCount={0}", _CallCount);
			Console.WriteLine("      Session={0}", OperationContext.Current.SessionId);
			return _CallCount;
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Singleton Service Test Sample......");

			// Singleton 서비스 인스턴스 생성
			SingletonService serviceInstance = new SingletonService();
			// 서비스 호스트와 종점 설정은 configuration을 사용한다.
			ServiceHost host = new ServiceHost(serviceInstance);
			host.Open();
			Console.WriteLine("Press any key to stop the service...");
			Console.ReadKey(true);
			host.Close();
			Console.WriteLine("ServiceHost is closed......");
			serviceInstance.Dispose();
		}
	}
}
