﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;

namespace CallbackTestService
{
	[ServiceContract]
	interface ITestCallbackContract
	{
		[OperationContract]
		void OnCallback(string message);
	}

	[ServiceContract(CallbackContract = typeof(ITestCallbackContract))]
	interface ITestContract
	{
		[OperationContract]
		void TestMethod();
	}

	[ServiceBehavior(InstanceContextMode = InstanceContextMode.PerCall,
		ConcurrencyMode = ConcurrencyMode.Reentrant, IncludeExceptionDetailInFaults=true)]
	class TestService : ITestContract
	{
		public void TestMethod()
		{
			Console.WriteLine("TestMethod() called...");
			OperationContext ctx = OperationContext.Current;
			ITestCallbackContract callback = ctx.GetCallbackChannel<ITestCallbackContract>();
			Console.WriteLine("Callback to {0}", ctx.RequestContext.RequestMessage.Headers.ReplyTo.Uri);
			callback.OnCallback("From TestService.TestMethod");
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Simple Callback Test Service Sample......");

			ServiceHost host = new ServiceHost(typeof(TestService),
				new Uri("http://localhost/wcf/example/callback/test"), 
				new Uri("net.tcp://localhost/wcf/example/callback/test"));
			WSDualHttpBinding wsDualHttpBinding = new WSDualHttpBinding();
			NetTcpBinding netTcpBinding = new NetTcpBinding();
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();

			wsDualHttpBinding.Security.Mode = WSDualHttpSecurityMode.None;
			netTcpBinding.Security.Mode = SecurityMode.None;
			netTcpBinding.ReliableSession.Enabled = false;
			behavior.HttpGetEnabled = true;

			host.AddServiceEndpoint(typeof(ITestContract), wsDualHttpBinding, "");
			host.AddServiceEndpoint(typeof(ITestContract), netTcpBinding, "");
			host.Description.Behaviors.Add(behavior);

			host.Open();
			Console.WriteLine("Press any key to shuthdown the service...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
